# coding: utf-8
"""
Nel fare questo modulo ho incontrato alcuni problemi irrisolti:

1. NullBooleanField: non riesco ad annullare un campo che sia
   già True or False. Es.: transactional  => occore usare ''

2. JsonField: mi tocca aggiungere json.dumps quando creo e json.loads
   quando deserializzo. Non mi pare molto logico

"""
from __future__ import unicode_literals

import logging

import requests
from requests.auth import HTTPBasicAuth
from requests.exceptions import HTTPError
import six


logger = logging.getLogger('jmb')

#from jmb.tools.main import settings

class MailingApiError(Exception):
    pass


class ValidationError(Exception):
    pass


class Resource(object):
    key = ""
    SLUG_FIELD = 'name'
    HUMAN_REPR = "{%s}" % SLUG_FIELD

    def __init__(self, base_uri, username=None, password=None, auth=None):

        self.base_uri = base_uri
        if auth:
            self.auth = auth
        else:
            self.auth = HTTPBasicAuth(username=username, password=password)


    def raise_for_status(self):
        """Raises stored :class:`HTTPError`, if one occurred."""
        ## REST stores error message in content!!!
        http_error_msg = ''
        if isinstance(self.response.reason, bytes):
            # We attempt to decode utf-8 first because some servers
            # choose to localize their reason strings. If the string
            # isn't utf-8, we fall back to iso-8859-1 for all other
            # encodings. (See PR #3538)
            try:
                reason = self.response.reason.decode('utf-8')
            except UnicodeDecodeError:
                reason = self.response.reason.decode('iso-8859-1')
        else:
            reason = self.response.reason

        if 400 <= self.response.status_code < 500:
            http_error_msg = u'%s Client Error: %s for url: %s' % (
                self.response.status_code, reason, self.uri)

        elif 500 <= self.response.status_code < 600:
            http_error_msg = u'%s Server Error: %s for url: %s' % (
                self.response.status_code, reason, self.uri)

        ## if response.content == b'', cannot be serialized!
        msg = '%s %s' % (http_error_msg, self.response.content and self.response.json())

        if http_error_msg:
            raise HTTPError(msg, response=self.response)

    @property
    def uri(self):
        return "%s/%s" % (self.base_uri, self.key)

    def add_or_update(self, data):
        """Add an entity if not already present
        """

        uri = self.uri

        try:
            item_data = self.get(data[self.SLUG_FIELD])
            update_uri = "{}{}/".format(self.uri, data[self.SLUG_FIELD])
            item_data.update(data)
            return self.request('patch', update_uri, json=item_data)
        except Exception as e:
            if not hasattr(self, 'response'):  # no connection
                raise
            if self.response.status_code == 404:
                return self.request('post', uri, json=data)
            else:
                raise

    def request(self, method, uri, *args, **kwargs):
        self.response = getattr(requests, method)(uri, auth=self.auth, *args, **kwargs)
        self.raise_for_status()
        try:
            return self.response.json()
        except:
            return None


    def get(self, slug=None):
        """Retrieve an entity using slug or pk according to api"""

        uri = "{}{}/".format(self.uri, slug)
        return self.request('get', uri)

    def list(self, only_slug=False):
        """Retrieve all entity """

        if only_slug:
            return [item[self.SLUG_FIELD] for item in self.request('get', self.uri)]
        else:
            return self.request('get', self.uri)

    def human_list(self, only_slug=False):
        """Retrieve all entity """

        return [self.HUMAN_REPR.format(**item) for item in self.request('get', self.uri)]

    def delete(self, slug):
        uri = "{}{}/".format(self.uri, slug)
        return self.request('delete', uri)



class Server(Resource):
    key = 'servers/'

class Key(Resource):
    key = 'keys/'

class App(Resource):
    key = 'apps/'

class Project(Resource):
    key = 'projects/'
    HUMAN_REPR = '{name} / {server}'

class AppProject(Resource):
    key = 'appproject/'
    HUMAN_REPR = '{project} {app}-{version}'
    SLUG_FIELD = 'id'

    def get_by_slugs(self, project, app):
        """Retrieve an entity using slug or pk according to api

        :arg project: the project name
        :arg app: the app name
        """

        uri = "{}get_by_slugs/?project={project}&app={app}".format(self.uri, **locals())
        return self.request('get', uri)

    def add_or_update_by_slugs(self, project, app, data):
        """Add an entity if not already present
        """

        uri = self.uri

        try:
            item_data = self.get_by_slugs(project, app)
            update_uri = "{}{}/".format(self.uri, item_data[self.SLUG_FIELD])
            item_data.update(data)
            return self.request('patch', update_uri, json=item_data)
        except Exception as e:
            if self.response.status_code == 404:
                return self.request('post', uri, json=data)
            else:
                raise

class StatsClient(object):

    def __init__(self, base_uri=None, username=None, password=None):

        self.base_uri = base_uri
        self.auth = HTTPBasicAuth(username=username, password=password)
        self.servers = Server(self.base_uri, auth=self.auth)
        self.keys = Key(self.base_uri, auth=self.auth)
        self.apps = App(self.base_uri, auth=self.auth)
        self.projects = Project(self.base_uri, auth=self.auth)
        self.appproject = AppProject(self.base_uri, auth=self.auth)

    def get(self, url='/'):
        url = self.base_uri + url
        self.resp = requests.get(url, auth=self.auth)
        if self.resp.ok:
            return self.resp.json()
        else:
            self.resp.raise_for_status()

    def test(self):
        return self.get()

    def post(self, url, data):
        url = self.base_uri + url
        self.resp = requests.post(url, json=data, auth=self.auth)
        if self.resp.ok:
            return self.resp.json()
        else:
            self.resp.raise_for_status()

    def delete(self, url, pk):
        url = self.base_uri + url + pk
        self.resp = requests.delete(url, auth=self.auth)
        self.resp.raise_for_status()

stats_client = StatsClient()
